#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
#include <time.h>
#include "vector.h"
#include "graphnet.h"

#define TRUE  1
#define FALSE 0

MATRIX  out0;                 /* input layer */
MATRIX  out2;                 /* output layer */
MATRIX  Weights;              /* weights for imput to output layer */
MATRIX  Euclidea;	      /* Distancia Euclidea de cada patron */
VECTOR  Ganadoras;            /* Ganadora de cada patron */
VECTOR  Dead;		      /* Neuronas muertas */

int nInputNodes;
int nOutputNodes;
int nPatterns;
int p;         /* indice del numero de patron */
int Side;      /* tamano del lado del rectangulo de salida*/
int j, 	       /* Indice de salida */
    i;         /* Indice de entrada */
char *argv;
int argc;

/* Prototipos de las funciones utilizadas */

void DisplayNet();
void ResetScreen();
void readweights(char *szWeights);
void readpatterns(char *szPattern);
void normalizeweights(int j);
void readactivations(char *szResults);
void readdead(char *fich_dead);

#include "inicgraf.inc"
#include "fungraf.inc"

void main(int argc, char *argv[])
{
FILE *fpmapa;
char fich_pat[50],
     fich_pesos[50],
     fich_result[50],
     fich_dead[50];

if (argc < 2 || argc > 2)
	{
	printf("\nAYUDA: MAPA fichero\n");
	exit(1);
	}
/* Abrir el fichero de entrada */
if ((fpmapa = fopen(argv[1], "r")) == NULL)
{
	printf("ERROR: al abrir el fichero: ", argv[1]);
	exit(1);
}

/* Leer el fichero de entrada */
fscanf(fpmapa, "%d %d %d %s %s %s %s",
       &nPatterns,                /* Patrones */
       &nInputNodes,              /* Nodos de entrada */
       &nOutputNodes,             /* Nodos de salida */
       &fich_pat,                 /* fichero de patrones */
       &fich_pesos,               /* fichero de salida  */
       &fich_result, 		  /* fichero de resultados */
       &fich_dead);		  /* fichero de neuronas muertas */

/* Capturar memoria para las matrices y vectores */
AllocateMatrix(&out0,  nPatterns,     nInputNodes);
AllocateMatrix(&out2,  nPatterns,     nOutputNodes);
AllocateMatrix(&Euclidea,  nPatterns,     nOutputNodes);
AllocateMatrix(&Weights, nOutputNodes, nInputNodes);
AllocateVector(&Ganadoras,nPatterns);
AllocateVector(&Dead, nOutputNodes);
/* Leo el fichero de patrones */
readpatterns(fich_pat);

/* Leo el fichero de pesos */
readweights(fich_pesos);

/* Leo el fichero  de resultados (activations) */
readactivations(fich_result);

/* Leo el fichero de neuronas muertas */
readdead(fich_dead);

/* Lado del mapa de kohonen */
Side = (int) sqrt ( (double) nOutputNodes);

/* Se calcula la ganadora y la distancia Euclidea */
for (p = 0; p < nPatterns; ++p)
	{
	float MinimunDistance = 0;
	int Winner;
	for (j = 0 ; j < nOutputNodes; j++)
		{
		float EuclideanDistance = 0;
		for (i = 0 ; i < nInputNodes; i++)
			{
			float d = out0[p][i] - Weights[j][i];
			EuclideanDistance += d * d;
			}
		Euclidea[p][j] = EuclideanDistance;
		if (j==0 || EuclideanDistance <= MinimunDistance)
			{
			MinimunDistance = EuclideanDistance;
			Winner = j;
			}
		}
	Ganadoras[p] = Winner;
	}

/* Inicializa el entorno grfico */
inicgraf();

/* Dibuja del mapa */
dibcontrol(nPatterns, Side, out2, Euclidea, Ganadoras, Dead);

/* Liberar memoria de matrices y vectores */
FreeMatrix(out0, nPatterns);
FreeMatrix(Weights, nOutputNodes);
FreeMatrix(Euclidea, nPatterns);
FreeMatrix(out2, nPatterns);
free(Ganadoras);
free(Dead);
}

/*--------------------------- normalizeweights ----------------------------*/
void normalizeweights(int j)
{
int i;
float d = 0;

for (i = 0 ; i < nInputNodes; i++)
	{
	float r = Weights[j][i];
	d += r * r;
	}
d = sqrt((double) d);

for (i = 0 ; i < nInputNodes; i++)
	Weights[j][i] /= d;

}

/*------------------------------ readdead ---------------------------------*/
void readdead(char *fich_dead)
{
int j;
FILE *fpdead;

if (strcmp(strupr(fich_dead), "NINGUNO") ==  0) /* Si no hay fichero de neuronas muertas */
	{
	for (j = 0 ; j < nOutputNodes; j++)
		Dead[j] = FALSE;
	return;
	}
if (( fpdead = fopen(fich_dead,"r")) == NULL)
	{
	fprintf(stderr, "can't open file %s\n",fich_dead);
	exit(1);
	}

for (j = 0 ; j < nOutputNodes; j++)
	 fscanf(fpdead, "%f", &Dead[j]);
fclose(fpdead);
}

/*--------------------------- readweights ---------------------------------*/
void readweights(char *szWeights)
{
int i;
int j;
FILE *fpWeights;    /*  initial weight file  */

if (( fpWeights = fopen(szWeights,"r")) == NULL)
	{
	fprintf(stderr, "can't open file %s\n",szWeights);
	exit(1);
	}
/*  Leer los pesos de las conneciones imput : output */
for (j = 0 ; j < nOutputNodes; j++)
	{
	for (i = 0 ; i < nInputNodes; i++)
		{
		fscanf(fpWeights, "%f", &Weights[j][i]);
		}
	normalizeweights(j);
	}
fclose(fpWeights);
}

/*--------------------------- readpatterns --------------------------------*/
void readpatterns(char *szPattern)
{
int i;
FILE *fpPattern;    /*  fichero de los patrones de entrada  */

if (( fpPattern = fopen(szPattern,"r")) == NULL)
	{
	fprintf(stderr, "can't open file %s\n",szPattern);
	exit(1);
	}
/*  Leer los pesos de las conneciones imput : output */
for (p = 0 ; p < nPatterns; p++)
	 {
	float d = 0;
	for (i = 0 ; i < nInputNodes; i++)
		{
		float r;
		if (fscanf(fpPattern, "%f", &r) != 1)
		goto ALLPATTERNSREAD;
		out0[p][i] = r;
		d += r * r;
		}
	d = sqrt((double)d);
	/* Normaliza los Patrones de Entrada */
	for (i = 0; i < nInputNodes; i++)
		out0[p][i] /=d;
	 }
ALLPATTERNSREAD:
fclose(fpPattern);

if (p < nPatterns)
	{
	fprintf(stderr,"%d  de  %d patrones leidos \n",p,nPatterns);
	nPatterns = p;
	}
}
/*--------------------------- redactivations ------------------------------*/
void readactivations(char *szResults)
{
int j;
FILE * fpResults;
if (( fpResults = fopen(szResults,"r")) == NULL)
	{
	fprintf(stderr, "can't open file %s\n",szResults);
	fpResults = stderr;
	}
for (p = 0 ; p < nPatterns; p++)
	{
	for (j = 0 ; j < nOutputNodes; j++)
		fscanf(fpResults, "%f\n", &out2[p][j]);
	}
fclose(fpResults);
}


/*--------------------------- AllocateVector ------------------------------*/
void AllocateVector(VECTOR *Vector, int nCols)
{
if ((*Vector = (VECTOR) calloc(nCols, sizeof(float))) == NULL)
	{
	fprintf(stderr, " Sorry Not enough memory for nodes !\n");
	exit(1);
	}
}

/*--------------------------- AllocateColumns -----------------------------*/
void AllocateColumns(PFLOAT Matrix[], int nRows, int nCols)
{
int i;
for (i = 0; i < nRows; i++)
	 AllocateVector(&Matrix[i], nCols);
}

/*--------------------------- AllocateMatrix ------------------------------*/
void AllocateMatrix(MATRIX *Pmatrix,int nRows, int nCols)
{
if ((*Pmatrix = (MATRIX) calloc(nRows, sizeof(PFLOAT))) ==
		 NULL)
	{
	fprintf(stderr, " Sorry Not enough memory for nodes !\n");
	exit(1);
	}
AllocateColumns(*Pmatrix, nRows, nCols);
}

/*--------------------------- FreeMatrix ----------------------------------*/
void FreeMatrix(MATRIX Matrix, int nRows)
{
int i;
for (i = 0; i < nRows; i++)
  free(Matrix[i]);
free(Matrix);
}

/*--------------------------- ResetScreen ---------------------------------*/
void  ResetScreen()
{
borrar(1,1,getmaxx(),getmaxy(), BLACK);
closegraph();
}

/*--------------------------- DisplayNet ----------------------------------*/
void DisplayNet()
{
dibcontrol(nPatterns, Side, out2, Euclidea, Ganadoras, Dead);
}
